// !hpf$ processors p(6)
// !hpf$ template t(20, 20)
// !hpf$ distribute t(*, block) onto p
// 
//   integer a(5)
//   real b(4, 5)
//   real c(5, 4)
// 
// !hpf$ align a(i) with t(*,i)
// !hpf$ align b(j, i) with t(5 * j, 4 * i)
// !hpf$ align c(i, *) with t(15 - 2 * i, 3)
// 
//   integer i, j
// 
//   do i = 1, 5
//     a(i) = 10 * i
//   enddo
// 
//   print "(' a...')"
//   do i = 1, 5
//     print "(i4)", a(i)
//   enddo
// 
//   b = spread(a, 1, 4)
//   
//   print "(' spread...')"
//   do j = 1, 4
//     print "(10f7.2)", b(j, : )
//   enddo
// 
//   do i = 1, 5
//     do j = 1, 4
//       b(j, i) = 10 * j + i
//     enddo
//   enddo
// 
//   print "(' b...')"
//   do j = 1, 4
//     print "(10f7.2)", b(j, : )
//   enddo
// 
//   c = transpose(b)
// 
//   print "(' transpose...')"
//   do i = 1, 5
//     print "(10f7.2)", c(i, : )
//   enddo
// 
// end

#include "ad++.h"

int main(int argc, char* argv[]) {
  AdlibInit(argc, argv) ;

  Procs1 p(6) ;

  Range t1(20) ;
  BlockRange t2(20, p.dim(0)) ;

  Range u = t2.subrng(5, 0) ;
  Array1<int> a(u, p) ;

  Range x = t1.subrng(4, 4, 5) ;
  Range y = t2.subrng(5, 3, 4) ;
  Array2<float> b(x, y, p) ;

  Range z = t1.subrng(5, 12, -2) ;
  Group q = p / t2(2) ;
  Array2<float> c(z, 4, q) ;

  ON(p) {
    Index i(u) ;
    OVERALL(i) {
      a(i) = 10 * (i + 1) ;
    } ALLOVER(i) ;
  } NO(p) ;

  gprintf(" a...\n") ;
  aprintf("%4d\n", a) ;

  Array1<int> tmp1(y, p) ;
  remap(tmp1, a) ;

  Array2<int> tmp2(x, y, p) ;  // Need `tmp2' due to type conversion.
  spread(tmp2, tmp1, 0) ;
  ON(p) {
    Index i(x), j(y) ;
    OVERALL(i) {
      OVERALL(j) {
	b(i, j) = tmp2(i, j) ;  // Type conversion
      } ALLOVER(j) ;
    } ALLOVER(i) ;
  } NO(p) ;

  gprintf(" spread...\n") ;
  aprintf("%7.2f%5N", b) ;

  ON(p) {
    Index i(x), j(y) ;
    OVERALL(i) {
      OVERALL(j) {
	b(i, j) = 10 * (i + 1) + j + 1 ;
      } ALLOVER(j) ;
    } ALLOVER(i) ;
  } NO(p) ;

  gprintf(" b...\n") ;
  aprintf("%7.2f%5N", b) ;

  transpose(c, b) ;

  gprintf(" transpose...\n") ;
  aprintf("%7.2f%4N", c) ;

  AdlibFinalize() ;
}


